@@ -149,7 +149,7 @@ fm_lattice_2d <- function(...) {
   UseMethod("fm_lattice_2d")
 }
 
-#' Lattice grids for inla.mesh
+#' Lattice grids for fm_mesh_2d
 #'
 #' Construct a lattice grid for [fm_mesh_2d()]
 #'
@@ -166,7 +166,8 @@ fm_lattice_2d <- function(...) {
 #' @param dims the size of the grid, length 2 vector
 #' @param units One of `c("default", "longlat", "longsinlat", "mollweide")`
 #' or NULL (equivalent to `"default"`).
-#' @param crs An optional `fm_crs`, `sf::st_crs`, or `sp::CRS` object
+#' @param crs An optional `fm_crs`, `sf::st_crs`, or `sp::CRS` object,
+#'   denoting the CRS info for the x-y grid.
 #' @returns An `fm_lattice_2d` object with elements
 #' \describe{
 #' \item{dims}{integer vector}
@@ -175,7 +176,8 @@ fm_lattice_2d <- function(...) {
 #' \item{loc}{matrix of `(x, y)` values or `(x, y, z)` values. May be altered by
 #' [fm_transform()]}
 #' \item{segm}{`fm_segm` object}
-#' \item{crs}{`fm_crs` object or `NULL`}
+#' \item{crs}{`fm_crs` object for `loc`, or `NULL`}
+#' \item{crs0}{`fm_crs` object for `(x,y)`, or `NULL`}
 #' }
 #' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
 #' @seealso [fm_mesh_2d()]
@@ -308,8 +310,20 @@ fm_lattice_2d.default <- function(
     crs = crs
   )
 
-  lattice <- list(dims = dims, x = x, y = y, loc = loc, segm = segm, crs = crs)
-  class(lattice) <- c("fm_lattice_2d", "inla.mesh.lattice")
+  crs0 <- crs
+
+  lattice <- structure(
+    list(
+      dims = dims,
+      x = x,
+      y = y,
+      loc = loc,
+      segm = segm,
+      crs = crs,
+      crs0 = crs0
+    ),
+    class = c("fm_lattice_2d", "inla.mesh.lattice")
+  )
   return(lattice)
 }
 
