@@ -124,8 +124,8 @@ fm_mesh_1d <- function(loc,
 
   n <- length(loc)
 
-  if ((degree < 0) || (degree > 2)) {
-    stop(paste("'degree' must be 0, 1, or 2.  'degree=",
+  if ((degree < 0) || (degree > 3)) {
+    stop(paste("'degree' must be 0, 1, 2, 3.  'degree=",
       degree,
       "' is not supported.",
       sep = ""
@@ -138,17 +138,26 @@ fm_mesh_1d <- function(loc,
 
 
   ## Number of basis functions
-  if (degree == 0) {
-    basis.reduction <- c(0, 1, 0, 1 / 2) ## neu, dir, free, cyclic
-  } else if (degree == 1) {
-    basis.reduction <- c(0, 1, 0, 1 / 2) ## neu, dir, free, cyclic
+  stopifnot(degree >= 0)
+  stopifnot(degree <= 3)
+  m_free <- n + c(0, 0, 1, 2)
+  # How many more basis functions at each end, compared with the number of
+  # knots; for cyclic, is only applied once:
+  m_adjust <- list(
+    c(0, -1, 0, 0), ## neu, dir, free, cyclic
+    c(0, -1, 0, 0), ## neu, dir, free, cyclic
+    c(-1, -1, 0, -1), ## neu, dir, free, cyclic
+    c(-1, -1, 0, -2) ## neu, dir, free, cyclic
+  )
+  if (cyclic) {
+    m <- m_free[degree + 1] + m_adjust[[degree + 1]][4]
   } else {
-    basis.reduction <- c(1, 1, 0, 1)
+    i1 <- pmatch(boundary[1], boundary.options)
+    i2 <- pmatch(boundary[2], boundary.options)
+    m <- m_free[degree + 1] +
+      m_adjust[[degree + 1]][i1] +
+      m_adjust[[degree + 1]][i2]
   }
-  m <- (n + cyclic + (degree == 2) * 1
-    - basis.reduction[pmatch(boundary[1], boundary.options)]
-    - basis.reduction[pmatch(boundary[2], boundary.options)])
-  ## if (m < 1+max(1,degree)) {
   if (m < 1L) {
     stop("Degree ", degree,
       " meshes must have at least ", 1L,
@@ -164,7 +173,15 @@ fm_mesh_1d <- function(loc,
       mid <- mid[-1]
     }
     if (boundary[2] == "dirichlet") {
-      mid <- mid[-(m + 1)]
+      mid <- mid[-length(mid)]
+    }
+  } else if (degree == 3) {
+    mid <- loc
+    if (boundary[1] == "dirichlet") {
+      mid <- mid[-1]
+    }
+    if (boundary[2] == "dirichlet") {
+      mid <- mid[-length(mid)]
     }
   } else { ## degree==2
     if (cyclic) {
@@ -204,16 +221,16 @@ fm_mesh_1d <- function(loc,
       class = c("fm_mesh_1d", "inla.mesh.1d")
     )
 
-  if (degree < 2) {
+  if (degree != 2) {
     mesh$idx$loc <-
-      fm_bary(mesh, loc.orig, method = "nearest")$t[, 1]
+      fm_bary(mesh, loc.orig, method = "nearest")$index
   } else {
     if (length(mid) >= 2) {
       mesh$idx$loc <-
         fm_bary(fm_mesh_1d(mid, degree = 0),
           loc.orig,
           method = "nearest"
-        )$t[, 1]
+        )$index
     } else {
       mesh$idx$loc <- rep(1, length(loc.orig))
     }
