@@ -1,7 +1,7 @@
 test_that("Discrete integration", {
   domain <- 2:5
   samplers <- 3:7
-  ips_ <- data.frame(x = 3:5, weight = rep(1, 3), .block = 1L:3L)
+  ips_ <- tibble::tibble(x = 3:5, weight = rep(1, 3), .block = 1L:3L)
 
   ips <- fm_int(domain, samplers = samplers)
   expect_identical(ips, ips_)
@@ -24,7 +24,7 @@ test_that("Continuous integration", {
   domain <- fm_mesh_1d(2:5)
 
   samplers <- c(3, 5)
-  ips_ <- data.frame(
+  ips_ <- tibble::tibble(
     x = c(3:5, 3.5, 4.5),
     weight = c(1 / 6, 1 / 3, 1 / 6, 2 / 3, 2 / 3),
     .block = 1L
@@ -35,6 +35,14 @@ test_that("Continuous integration", {
   ips <- ips[order(ips$x), ]
   expect_identical(ips, ips_)
 
+  ips_bary_ <- ips_
+  ips_bary_$x <- fm_bary(domain, ips_$x)
+
+  ips_bary <- fm_int(domain, samplers = samplers, format = "bary")
+  ips_bary_ <- ips_bary_[order(ips_bary_$x$index, ips_bary_$x$where[, 2]), ]
+  ips_bary <- ips_bary[order(ips_bary$x$index, ips_bary$x$where[, 2]), ]
+  expect_identical(ips_bary, ips_bary_)
+
   # Check blockwise integration
   samplers <- rbind(c(3, 5), c(2, 4.5))
   ips <- fm_int(domain, samplers = samplers)
@@ -45,7 +53,7 @@ test_that("Continuous integration", {
   domain <- fm_mesh_1d(2:5, degree = 2)
 
   samplers <- c(3, 5)
-  ips_ <- data.frame(
+  ips_ <- tibble::tibble(
     x = c(3:5, 3.5, 4.5),
     weight = c(1 / 6, 1 / 3, 1 / 6, 2 / 3, 2 / 3),
     .block = 1L
